defmodule WildWeb.BookReadLive.FormComponent do
  use WildWeb, :live_component

  alias Wild.BookReads

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        {@title}
        <:subtitle>Use this form to manage book_read records in your database.</:subtitle>
      </.header>

      <.simple_form
        for={@form}
        id="book_read-form"
        phx-target={@myself}
        phx-change="validate"
        phx-submit="save"
      >
        <.input field={@form[:rating]} type="number" label="Rating" />
        <.input field={@form[:thoughts]} type="text" label="Thoughts" />
        <.input field={@form[:read_start]} type="datetime-local" label="Read start" />
        <.input field={@form[:read_finish]} type="datetime-local" label="Read finish" />
        <.input field={@form[:progress]} type="number" label="Progress" />
        <:actions>
          <.button phx-disable-with="Saving...">Save Book read</.button>
        </:actions>
      </.simple_form>
    </div>
    """
  end

  @impl true
  def update(%{book_read: book_read} = assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign_new(:form, fn ->
       to_form(BookReads.change_book_read(book_read))
     end)}
  end

  @impl true
  def handle_event("validate", %{"book_read" => book_read_params}, socket) do
    changeset = BookReads.change_book_read(socket.assigns.book_read, book_read_params)
    {:noreply, assign(socket, form: to_form(changeset, action: :validate))}
  end

  def handle_event("save", %{"book_read" => book_read_params}, socket) do
    save_book_read(socket, socket.assigns.action, book_read_params)
  end

  defp save_book_read(socket, :edit, book_read_params) do
    case BookReads.update_book_read(socket.assigns.book_read, book_read_params) do
      {:ok, book_read} ->
        notify_parent({:saved, book_read})

        {:noreply,
         socket
         |> put_flash(:info, "Book read updated successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign(socket, form: to_form(changeset))}
    end
  end

  defp save_book_read(socket, :new, book_read_params) do
    case BookReads.create_book_read(book_read_params) do
      {:ok, book_read} ->
        notify_parent({:saved, book_read})

        {:noreply,
         socket
         |> put_flash(:info, "Book read created successfully")
         |> push_patch(to: socket.assigns.patch)}

      {:error, %Ecto.Changeset{} = changeset} ->
        {:noreply, assign(socket, form: to_form(changeset))}
    end
  end

  defp notify_parent(msg), do: send(self(), {__MODULE__, msg})
end
